@@ -3,7 +3,7 @@
 /**
  +-------------------------------------------------------------------------+
  | Roundcube Webmail setup tool                                            |
- | Version 1.5.2                                                           |
+ | Version 1.6-git                                                         |
  |                                                                         |
  | Copyright (C) The Roundcube Dev Team                                    |
  |                                                                         |
@@ -71,7 +71,7 @@ if (isset($_GET['_getconfig'])) {
 
 if (
     $RCI->configured
-    && ($RCI->getprop('enable_installer') || $_SESSION['allowinstaller']) &&
+    && ($RCI->getprop('enable_installer') || !empty($_SESSION['allowinstaller'])) &&
     !empty($_GET['_mergeconfig'])
 ) {
     $filename = 'config.inc.php';
