@@ -3,8 +3,8 @@
 /**
  * Class for operations on Sieve scripts
  *
- * Copyright (C) 2008-2011, The Roundcube Dev Team
- * Copyright (C) 2011, Kolab Systems AG
+ * Copyright (C) The Roundcube Dev Team
+ * Copyright (C) Kolab Systems AG
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,15 +22,16 @@
 
 class rcube_sieve_script
 {
-    public $content = array();      // script rules array
+    public $content = [];      // script rules array
 
-    private $vars      = array();   // "global" variables
-    private $prefix    = '';        // script header (comments)
-    private $supported = array(     // supported Sieve extensions:
+    private $vars      = [];   // "global" variables
+    private $prefix    = '';   // script header (comments)
+    private $supported = [     // supported Sieve extensions:
         'body',                     // RFC5173
         'copy',                     // RFC3894
         'date',                     // RFC5260
         'duplicate',                // RFC7352
+        'editheader',               // RFC5293
         'enotify',                  // RFC5435
         'envelope',                 // RFC5228
         'ereject',                  // RFC5429
@@ -39,6 +40,7 @@ class rcube_sieve_script
         'imap4flags',               // RFC5232
         'include',                  // RFC6609
         'index',                    // RFC5260
+        'mime',                     // RFC5703 (except: foreverypart/break, enclose, extracttext)
         'notify',                   // RFC5435
         'regex',                    // draft-ietf-sieve-regex-01
         'reject',                   // RFC5429
@@ -47,16 +49,17 @@ class rcube_sieve_script
         'vacation',                 // RFC5230
         'vacation-seconds',         // RFC6131
         'variables',                // RFC5229
-        // @TODO: spamtest+virustest, mailbox
-    );
+        'spamtest',                 // RFC3685 (not RFC5235 with :percent argument)
+        // @TODO: virustest, mailbox
+    ];
 
     /**
      * Object constructor
      *
-     * @param  string  Script's text content
-     * @param  array   List of capabilities supported by server
+     * @param string $script       Script's text content
+     * @param array  $capabilities List of capabilities supported by server
      */
-    public function __construct($script, $capabilities=array())
+    public function __construct($script, $capabilities = [])
     {
         $capabilities = array_map('strtolower', (array) $capabilities);
 
@@ -76,8 +79,7 @@ class rcube_sieve_script
     /**
      * Adds rule to the script (at the end)
      *
-     * @param string Rule name
-     * @param array  Rule content (as array)
+     * @param array $content Rule content (as array)
      *
      * @return int The index of the new rule
      */
@@ -88,6 +90,13 @@ class rcube_sieve_script
         return count($this->content) - 1;
     }
 
+    /**
+     * Removes a rule from the script.
+     *
+     * @param int $index Rule index
+     *
+     * @return bool True on success, False otherwise
+     */
     public function delete_rule($index)
     {
         if (isset($this->content[$index])) {
@@ -98,15 +107,28 @@ class rcube_sieve_script
         return false;
     }
 
+    /**
+     * Get the script size - count of rules.
+     *
+     * @return int Count of rules
+     */
     public function size()
     {
         return count($this->content);
     }
 
+    /**
+     * Updates (replaces) a rule at specified index.
+     *
+     * @param int   $index   Rule index
+     * @param array $content Rule content (as array)
+     *
+     * @return bool True on success, False otherwise
+     */
     public function update_rule($index, $content)
     {
         // TODO: check this->supported
-        if ($this->content[$index]) {
+        if (isset($this->content[$index])) {
             $this->content[$index] = $content;
             return $index;
         }
@@ -121,7 +143,7 @@ class rcube_sieve_script
      * @param string $value Variable value
      * @param array  $mods  Variable modifiers
      */
-    public function set_var($name, $value, $mods = array())
+    public function set_var($name, $value, $mods = [])
     {
         // Check if variable exists
         for ($i=0, $len=count($this->vars); $i<$len; $i++) {
@@ -130,14 +152,15 @@ class rcube_sieve_script
             }
         }
 
-        $var = array_merge($mods, array('name' => $name, 'value' => $value));
+        $var = array_merge($mods, ['name' => $name, 'value' => $value]);
+
         $this->vars[$i] = $var;
     }
 
     /**
      * Unsets "global" variable
      *
-     * @param string $name  Variable name
+     * @param string $name Variable name
      */
     public function unset_var($name)
     {
@@ -151,9 +174,9 @@ class rcube_sieve_script
     }
 
     /**
-     * Gets the value of  "global" variable
+     * Gets the value of "global" variable
      *
-     * @param string $name  Variable name
+     * @param string $name Variable name
      *
      * @return string Variable value
      */
@@ -170,7 +193,7 @@ class rcube_sieve_script
     /**
      * Sets script header content
      *
-     * @param string $text  Header content
+     * @param string $text Header content
      */
     public function set_prefix($text)
     {
@@ -183,7 +206,7 @@ class rcube_sieve_script
     public function as_text()
     {
         $output = '';
-        $exts   = array();
+        $exts   = [];
         $idx    = 0;
 
         if (!empty($this->vars)) {
@@ -194,14 +217,14 @@ class rcube_sieve_script
             foreach ($this->vars as $var) {
                 if (empty($has_vars)) {
                     // 'variables' extension not supported, put vars in comments
-                    $output .= sprintf("# %s %s\n", $var['name'], $var['value']);
+                    $output .= sprintf("# %s %s\r\n", $var['name'], $var['value']);
                 }
                 else {
                     $output .= 'set ';
-                    foreach (array_diff(array_keys($var), array('name', 'value')) as $opt) {
+                    foreach (array_diff(array_keys($var), ['name', 'value']) as $opt) {
                         $output .= ":$opt ";
                     }
-                    $output .= self::escape_string($var['name']) . ' ' . self::escape_string($var['value']) . ";\n";
+                    $output .= self::escape_string($var['name']) . ' ' . self::escape_string($var['value']) . ";\r\n";
                 }
             }
         }
@@ -212,12 +235,12 @@ class rcube_sieve_script
         // rules
         foreach ($this->content as $rule) {
             $script    = '';
-            $tests     = array();
+            $tests     = [];
             $i         = 0;
 
             // header
             if (!empty($rule['name']) && strlen($rule['name'])) {
-                $script .= '# rule:[' . $rule['name'] . "]\n";
+                $script .= '# rule:[' . $rule['name'] . "]\r\n";
             }
 
             // constraints expressions
@@ -227,16 +250,29 @@ class rcube_sieve_script
                     switch ($test['test']) {
                     case 'size':
                         $tests[$i] .= ($test['not'] ? 'not ' : '');
-                        $tests[$i] .= 'size :' . ($test['type']=='under' ? 'under ' : 'over ') . $test['arg'];
+                        $tests[$i] .= 'size :' . ($test['type'] == 'under' ? 'under ' : 'over ') . $test['arg'];
+                        break;
+
+                    case 'spamtest':
+                        array_push($exts, 'spamtest');
+                        $tests[$i] .= ($test['not'] ? 'not ' : '');
+                        $tests[$i] .= $test['test'];
+
+                        $this->add_operator($test, $tests[$i], $exts);
+
+                        $tests[$i] .= ' ' . self::escape_string($test['arg']);
                         break;
 
                     case 'true':
-                        $tests[$i] .= ($test['not'] ? 'false' : 'true');
+                        $tests[$i] .= !empty($test['not']) ? 'false' : 'true';
                         break;
 
                     case 'exists':
-                        $tests[$i] .= ($test['not'] ? 'not ' : '');
-                        $tests[$i] .= 'exists ' . self::escape_string($test['arg']);
+                        $tests[$i] .= (!empty($test['not']) ? 'not ' : '') . 'exists';
+
+                        $this->add_mime($test, $tests[$i], $exts);
+
+                        $tests[$i] .= ' ' . self::escape_string($test['arg']);
                         break;
 
                     case 'header':
@@ -248,6 +284,10 @@ class rcube_sieve_script
                         $tests[$i] .= ($test['not'] ? 'not ' : '');
                         $tests[$i] .= $test['test'];
 
+                        if ($test['test'] == 'header') {
+                            $this->add_mime($test, $tests[$i], $exts);
+                        }
+
                         $this->add_index($test, $tests[$i], $exts);
                         $this->add_operator($test, $tests[$i], $exts);
 
@@ -264,7 +304,8 @@ class rcube_sieve_script
                         $tests[$i] .= ($test['not'] ? 'not ' : '');
                         $tests[$i] .= $test['test'];
 
-                        if ($test['test'] != 'envelope') {
+                        if ($test['test'] == 'address') {
+                            $this->add_mime($test, $tests[$i], $exts);
                             $this->add_index($test, $tests[$i], $exts);
                         }
 
@@ -329,11 +370,11 @@ class rcube_sieve_script
                     case 'duplicate':
                         array_push($exts, 'duplicate');
 
-                        $tests[$i] .= ($test['not'] ? 'not ' : '') . $test['test'];
+                        $tests[$i] .= (!empty($test['not']) ? 'not ' : '') . $test['test'];
 
-                        $tokens = array('handle', 'uniqueid', 'header');
+                        $tokens = ['handle', 'uniqueid', 'header'];
                         foreach ($tokens as $token)
-                            if ($test[$token] !== null && $test[$token] !== '') {
+                            if (isset($test[$token]) && $test[$token] !== '') {
                                 $tests[$i] .= " :$token " . self::escape_string($test[$token]);
                             }
 
@@ -369,7 +410,7 @@ class rcube_sieve_script
                 else {
                     $script .= $tests_str;
                 }
-                $script .= "\n{\n";
+                $script .= "\r\n{\r\n";
             }
 
             // action(s)
@@ -382,7 +423,7 @@ class rcube_sieve_script
                     case 'fileinto':
                         array_push($exts, 'fileinto');
                         $action_script .= 'fileinto ';
-                        if ($action['copy']) {
+                        if (!empty($action['copy'])) {
                             $action_script .= ':copy ';
                             array_push($exts, 'copy');
                         }
@@ -391,7 +432,7 @@ class rcube_sieve_script
 
                     case 'redirect':
                         $action_script .= 'redirect ';
-                        if ($action['copy']) {
+                        if (!empty($action['copy'])) {
                             $action_script .= ':copy ';
                             array_push($exts, 'copy');
                         }
@@ -401,16 +442,34 @@ class rcube_sieve_script
                     case 'reject':
                     case 'ereject':
                         array_push($exts, $action['type']);
-                        $action_script .= $action['type'].' '
-                            . self::escape_string($action['target']);
+                        $action_script .= $action['type'] . ' ' . self::escape_string($action['target']);
                         break;
 
                     case 'addflag':
                     case 'setflag':
                     case 'removeflag':
                         array_push($exts, $imapflags);
-                        $action_script .= $action['type'].' '
-                            . self::escape_string($action['target']);
+                        $action_script .= $action['type'] . ' ' . self::escape_string($action['target']);
+                        break;
+
+                    case 'addheader':
+                    case 'deleteheader':
+                        array_push($exts, 'editheader');
+                        $action_script .= $action['type'];
+                        if (!empty($action['index'])) {
+                            $action_script .= " :index " . intval($action['index']);
+                        }
+                        if (!empty($action['last']) && (!empty($action['index']) || $action['type'] == 'addheader')) {
+                            $action_script .= " :last";
+                        }
+                        if ($action['type'] == 'deleteheader') {
+                            $action['type'] = $action['match-type'] ?? null;
+                            $this->add_operator($action, $action_script, $exts);
+                        }
+                        $action_script .= " " . self::escape_string($action['name']);
+                        if ((is_string($action['value']) && strlen($action['value']) > 0) || (is_array($action['value']) && !empty($action['value']))) {
+                            $action_script .= " " . self::escape_string($action['value']);
+                        }
                         break;
 
                     case 'keep':
@@ -422,7 +481,7 @@ class rcube_sieve_script
                     case 'include':
                         array_push($exts, 'include');
                         $action_script .= 'include ';
-                        foreach (array_diff(array_keys($action), array('target', 'type')) as $opt) {
+                        foreach (array_diff(array_keys($action), ['target', 'type']) as $opt) {
                             $action_script .= ":$opt ";
                         }
                         $action_script .= self::escape_string($action['target']);
@@ -431,12 +490,27 @@ class rcube_sieve_script
                     case 'set':
                         array_push($exts, 'variables');
                         $action_script .= 'set ';
-                        foreach (array_diff(array_keys($action), array('name', 'value', 'type')) as $opt) {
+                        foreach (array_diff(array_keys($action), ['name', 'value', 'type']) as $opt) {
                             $action_script .= ":$opt ";
                         }
                         $action_script .= self::escape_string($action['name']) . ' ' . self::escape_string($action['value']);
                         break;
 
+                    case 'replace':
+                        array_push($exts, 'mime');
+                        $action_script .= 'replace';
+                        if (!empty($action['mime'])) {
+                            $action_script .= " :mime";
+                        }
+                        if (!empty($action['subject'])) {
+                            $action_script .= " :subject " . self::escape_string($action['subject']);
+                        }
+                        if (!empty($action['from'])) {
+                            $action_script .= " :from " . self::escape_string($action['from']);
+                        }
+                        $action_script .= ' ' . self::escape_string($action['replace']);
+                        break;
+
                     case 'notify':
                         array_push($exts, $notify);
                         $action_script .= 'notify';
@@ -447,10 +521,12 @@ class rcube_sieve_script
 
                         // Here we support draft-martin-sieve-notify-01 used by Cyrus
                         if ($notify == 'notify') {
-                            switch ($action['importance']) {
-                                case 1: $action_script .= " :high"; break;
-                                //case 2: $action_script .= " :normal"; break;
-                                case 3: $action_script .= " :low"; break;
+                            if (!empty($action['importance'])) {
+                                switch ($action['importance']) {
+                                    case 1: $action_script .= " :high"; break;
+                                    //case 2: $action_script .= " :normal"; break;
+                                    case 3: $action_script .= " :low"; break;
+                                }
                             }
 
                             // Old-draft way: :method "mailto" :options "email@address"
@@ -465,7 +541,7 @@ class rcube_sieve_script
                             unset($method);
                         }
 
-                        foreach (array('id', 'importance', 'method', 'options', 'from', 'message') as $n_tag) {
+                        foreach (['id', 'importance', 'method', 'options', 'from', 'message'] as $n_tag) {
                             if (!empty($action[$n_tag])) {
                                 $action_script .= " :$n_tag " . self::escape_string($action[$n_tag]);
                             }
@@ -487,29 +563,34 @@ class rcube_sieve_script
                         else if (!empty($action['days'])) {
                             $action_script .= " :days " . intval($action['days']);
                         }
-                        if (!empty($action['addresses']))
+                        if (!empty($action['addresses'])) {
                             $action_script .= " :addresses " . self::escape_string($action['addresses']);
-                        if (!empty($action['subject']))
+                        }
+                        if (!empty($action['subject'])) {
                             $action_script .= " :subject " . self::escape_string($action['subject']);
-                        if (!empty($action['handle']))
+                        }
+                        if (!empty($action['handle'])) {
                             $action_script .= " :handle " . self::escape_string($action['handle']);
-                        if (!empty($action['from']))
+                        }
+                        if (!empty($action['from'])) {
                             $action_script .= " :from " . self::escape_string($action['from']);
-                        if (!empty($action['mime']))
+                        }
+                        if (!empty($action['mime'])) {
                             $action_script .= " :mime";
+                        }
                         $action_script .= " " . self::escape_string($action['reason']);
                         break;
                     }
 
                     if ($action_script) {
                         $script .= !empty($tests) ? "\t" : '';
-                        $script .= $action_script . ";\n";
+                        $script .= $action_script . ";\r\n";
                     }
                 }
             }
 
             if ($script) {
-                $output .= $script . (!empty($tests) ? "}\n" : '');
+                $output .= $script . (!empty($tests) ? "}\r\n" : '');
                 $idx++;
             }
         }
@@ -524,11 +605,11 @@ class rcube_sieve_script
 
             sort($exts); // for convenience use always the same order
 
-            $output = 'require ["' . implode('","', $exts) . "\"];\n" . $output;
+            $output = 'require ["' . implode('","', $exts) . "\"];\r\n" . $output;
         }
 
         if (!empty($this->prefix)) {
-            $output = $this->prefix . "\n\n" . $output;
+            $output = $this->prefix . "\r\n\r\n" . $output;
         }
 
         return $output;
@@ -560,7 +641,7 @@ class rcube_sieve_script
     private function _parse_text($script)
     {
         $prefix   = '';
-        $options  = array();
+        $options  = [];
         $position = 0;
         $length   = strlen($script);
 
@@ -570,7 +651,7 @@ class rcube_sieve_script
             $rulename = '';
 
             // Comments
-            while ($script[$position] === '#') {
+            while (isset($script[$position]) && $script[$position] === '#') {
                 $endl = strpos($script, "\n", $position) ?: $length;
                 $line = substr($script, $position, $endl - $position);
 
@@ -614,7 +695,7 @@ class rcube_sieve_script
             // Simple commands
             else {
                 $rule = $this->_parse_actions($script, $position, ';');
-                if (!empty($rule[0]) && is_array($rule)) {
+                if (is_array($rule) && !empty($rule[0])) {
                     // set "global" variables
                     if ($rule[0]['type'] == 'set') {
                         unset($rule[0]['type']);
@@ -622,7 +703,7 @@ class rcube_sieve_script
                         unset($rule);
                     }
                     else {
-                        $rule = array('actions' => $rule);
+                        $rule = ['actions' => $rule];
                     }
                 }
             }
@@ -633,7 +714,7 @@ class rcube_sieve_script
         }
 
         if (!empty($prefix)) {
-            $this->prefix = trim($prefix);
+            $this->prefix = trim(preg_replace('/\r?\n/', "\r\n", $prefix));
         }
     }
 
@@ -699,7 +780,7 @@ class rcube_sieve_script
                 break;
 
             case 'size':
-                $test = array('test' => 'size', 'not' => $not);
+                $test = ['test' => 'size', 'not' => $not];
 
                 $test['arg'] = array_pop($tokens);
 
@@ -714,11 +795,21 @@ class rcube_sieve_script
                 $tests[] = $test;
                 break;
 
+            case 'spamtest':
+                $test = ['test' => 'spamtest', 'not' => $not];
+
+                $test['arg'] = array_pop($tokens);
+
+                $test += $this->test_tokens($tokens);
+
+                $tests[] = $test;
+                break;
+
             case 'header':
             case 'string':
             case 'address':
             case 'envelope':
-                $test = array('test' => $token, 'not' => $not);
+                $test = ['test' => $token, 'not' => $not];
 
                 $test['arg2'] = array_pop($tokens);
                 $test['arg1'] = array_pop($tokens);
@@ -737,7 +828,7 @@ class rcube_sieve_script
                 break;
 
             case 'body':
-                $test = array('test' => 'body', 'not' => $not);
+                $test = ['test' => 'body', 'not' => $not];
 
                 $test['arg'] = array_pop($tokens);
 
@@ -758,7 +849,7 @@ class rcube_sieve_script
 
             case 'date':
             case 'currentdate':
-                $test = array('test' => $token, 'not' => $not);
+                $test = ['test' => $token, 'not' => $not];
 
                 $test['arg']  = array_pop($tokens);
                 $test['part'] = array_pop($tokens);
@@ -782,7 +873,7 @@ class rcube_sieve_script
                 break;
 
             case 'duplicate':
-                $test = array('test' => $token, 'not' => $not);
+                $test = ['test' => $token, 'not' => $not];
 
                 for ($i=0, $len=count($tokens); $i<$len; $i++) {
                     if (!is_array($tokens[$i])) {
@@ -799,16 +890,17 @@ class rcube_sieve_script
                 break;
 
             case 'exists':
-                $tests[] = array('test' => 'exists', 'not' => $not,
-                    'arg'  => array_pop($tokens));
+                $test = ['test' => 'exists', 'not' => $not, 'arg'  => array_pop($tokens)];
+                $test += $this->test_tokens($tokens);
+                $tests[] = $test;
                 break;
 
             case 'true':
-                $tests[] = array('test' => 'true', 'not' => $not);
+                $tests[] = ['test' => 'true', 'not' => $not];
                 break;
 
             case 'false':
-                $tests[] = array('test' => 'true', 'not' => !$not);
+                $tests[] = ['test' => 'true', 'not' => !$not];
                 break;
             }
 
@@ -821,17 +913,17 @@ class rcube_sieve_script
         // ...and actions block
         $actions = $this->_parse_actions($content, $position);
 
-        if ($tests && $actions) {
-            $result = array(
+        if (!empty($tests) && $actions) {
+            return [
                 'type'     => $cond,
                 'tests'    => $tests,
                 'actions'  => $actions,
                 'join'     => $join,
                 'disabled' => $disabled,
-            );
+            ];
         }
 
-        return $result;
+        return null;
     }
 
     /**
@@ -845,7 +937,7 @@ class rcube_sieve_script
      */
     private function _parse_actions($content, &$position, $end = '}')
     {
-        $result = null;
+        $result = [];
         $length = strlen($content);
 
         while ($position < $length) {
@@ -855,6 +947,8 @@ class rcube_sieve_script
 
             switch ($token) {
             case 'if':
+            case 'else':
+            case 'elsif':
                 // nested 'if' conditions, ignore the whole rule (#5540)
                 $this->_parse_actions($content, $position);
                 continue 2;
@@ -862,60 +956,84 @@ class rcube_sieve_script
             case 'discard':
             case 'keep':
             case 'stop':
-                $result[] = array('type' => $token);
+                $result[] = ['type' => $token];
                 break;
 
             case 'fileinto':
             case 'redirect':
-                $action  = array('type' => $token, 'target' => array_pop($tokens));
-                $args    = array('copy');
+                $action  = ['type' => $token, 'target' => array_pop($tokens)];
+                $args    = ['copy'];
                 $action += $this->action_arguments($tokens, $args);
 
                 $result[] = $action;
                 break;
 
             case 'vacation':
-                $action  = array('type' => 'vacation', 'reason' => array_pop($tokens));
-                $args    = array('mime');
-                $vargs   = array('seconds', 'days', 'addresses', 'subject', 'handle', 'from');
+                $action  = ['type' => 'vacation', 'reason' => array_pop($tokens)];
+                $args    = ['mime'];
+                $vargs   = ['seconds', 'days', 'addresses', 'subject', 'handle', 'from'];
                 $action += $this->action_arguments($tokens, $args, $vargs);
 
                 $result[] = $action;
                 break;
 
+            case 'addheader':
+            case 'deleteheader':
+                $args = $this->test_tokens($tokens);
+                if ($token == 'deleteheader' && !empty($args['type'])) {
+                    $args['match-type'] = $args['type'];
+                }
+                if (($index = array_search(':last', $tokens)) !== false) {
+                    $args['last'] = true;
+                    unset($tokens[$index]);
+                }
+                $action = ['type' => $token, 'name' => array_shift($tokens), 'value' => array_shift($tokens)];
+
+                $result[] = $action + $args;
+                break;
+
             case 'reject':
             case 'ereject':
             case 'setflag':
             case 'addflag':
             case 'removeflag':
-                $result[] = array('type' => $token, 'target' => array_pop($tokens));
+                $result[] = ['type' => $token, 'target' => array_pop($tokens)];
                 break;
 
             case 'include':
-                $action  = array('type' => 'include', 'target' => array_pop($tokens));
-                $args    = array('once', 'optional', 'global', 'personal');
+                $action  = ['type' => 'include', 'target' => array_pop($tokens)];
+                $args    = ['once', 'optional', 'global', 'personal'];
                 $action += $this->action_arguments($tokens, $args);
 
                 $result[] = $action;
                 break;
 
             case 'set':
-                $action  = array('type' => 'set', 'value' => array_pop($tokens), 'name' => array_pop($tokens));
-                $args    = array('lower', 'upper', 'lowerfirst', 'upperfirst', 'quotewildcard', 'length');
+                $action  = ['type' => 'set', 'value' => array_pop($tokens), 'name' => array_pop($tokens)];
+                $args    = ['lower', 'upper', 'lowerfirst', 'upperfirst', 'quotewildcard', 'length'];
                 $action += $this->action_arguments($tokens, $args);
 
                 $result[] = $action;
                 break;
 
+            case 'replace':
+                $action  = ['type' => 'replace', 'replace' => array_pop($tokens)];
+                $args    = ['mime'];
+                $vargs   = ['subject', 'from'];
+                $action += $this->action_arguments($tokens, $args, $vargs);
+
+                $result[] = $action;
+                break;
+
             case 'require':
                 // skip, will be build according to used commands
-                // $result[] = array('type' => 'require', 'target' => array_pop($tokens));
+                // $result[] = ['type' => 'require', 'target' => array_pop($tokens)];
                 break;
 
             case 'notify':
-                $action     = array('type' => 'notify');
-                $priorities = array('high' => 1, 'normal' => 2, 'low' => 3);
-                $vargs      = array('from', 'id', 'importance', 'options', 'message', 'method');
+                $action     = ['type' => 'notify'];
+                $priorities = ['high' => 1, 'normal' => 2, 'low' => 3];
+                $vargs      = ['from', 'id', 'importance', 'options', 'message', 'method'];
                 $args       = array_keys($priorities);
                 $action    += $this->action_arguments($tokens, $args, $vargs);
 
@@ -929,7 +1047,7 @@ class rcube_sieve_script
                     }
                 }
 
-                $action['options'] = (array) $action['options'];
+                $action['options'] = isset($action['options']) ? (array) $action['options'] : [];
 
                 // Old-draft way: :method "mailto" :options "email@address"
                 if (!empty($action['method']) && !empty($action['options'])) {
@@ -949,7 +1067,7 @@ class rcube_sieve_script
             }
         }
 
-        return $result;
+        return !empty($result) ? $result : null;
     }
 
     /**
@@ -965,7 +1083,7 @@ class rcube_sieve_script
             array_push($exts, 'relational');
             array_push($exts, 'comparator-i;ascii-numeric');
         }
-        else if (!in_array($test['comparator'], array('i;octet', 'i;ascii-casemap'))) {
+        else if (!in_array($test['comparator'], ['i;octet', 'i;ascii-casemap'])) {
             array_push($exts, 'comparator-' . $test['comparator']);
         }
 
@@ -982,7 +1100,32 @@ class rcube_sieve_script
     {
         if (!empty($test['index'])) {
             array_push($exts, 'index');
-            $out .= ' :index ' . intval($test['index']) . ($test['last'] ? ' :last' : '');
+            $out .= ' :index ' . intval($test['index']) . (!empty($test['last']) ? ' :last' : '');
+        }
+    }
+
+    /**
+     * Add mime argument(s) to the test
+     */
+    private function add_mime($test, &$out, &$exts)
+    {
+        foreach (['mime', 'mime-anychild', 'mime-type', 'mime-subtype', 'mime-contenttype', 'mime-param'] as $opt) {
+            if (!empty($test[$opt])) {
+                $opt_name = str_replace('mime-', '', $opt);
+                if (empty($got_mime)) {
+                    $out .= ' :mime';
+                    $got_mime = true;
+                    array_push($exts, 'mime');
+                }
+
+                if ($opt_name != 'mime') {
+                    $out .= " :$opt_name";
+                }
+
+                if ($opt_name == 'param') {
+                    $out .= ' ' . self::escape_string($test[$opt]);
+                }
+            }
         }
     }
 
@@ -1017,29 +1160,35 @@ class rcube_sieve_script
      */
     private function test_tokens(&$tokens)
     {
-        $test   = array();
-        $result = array();
+        $test   = [];
+        $result = [];
 
         for ($i=0, $len=count($tokens); $i<$len; $i++) {
-            if (!is_array($tokens[$i]) && preg_match('/^:comparator$/i', $tokens[$i])) {
+            $token = is_array($tokens[$i]) ? null : $tokens[$i];
+            if ($token && preg_match('/^:comparator$/i', $token)) {
                 $test['comparator'] = $tokens[++$i];
             }
-            else if (!is_array($tokens[$i]) && preg_match('/^:(count|value)$/i', $tokens[$i])) {
-                $test['type'] = strtolower(substr($tokens[$i], 1)) . '-' . $tokens[++$i];
+            else if ($token && preg_match('/^:(count|value)$/i', $token)) {
+                $test['type'] = strtolower(substr($token, 1)) . '-' . $tokens[++$i];
             }
-            else if (!is_array($tokens[$i]) && preg_match('/^:(is|contains|matches|regex)$/i', $tokens[$i])) {
-                $test['type'] = strtolower(substr($tokens[$i], 1));
+            else if ($token && preg_match('/^:(is|contains|matches|regex)$/i', $token)) {
+                $test['type'] = strtolower(substr($token, 1));
             }
-            else if (!is_array($tokens[$i]) && preg_match('/^:index$/i', $tokens[$i])) {
+            else if ($token && preg_match('/^:(mime|anychild|type|subtype|contenttype|param)$/i', $token)) {
+                $token = strtolower(substr($token, 1));
+                $key   = $token == 'mime' ? $token : "mime-$token";
+                $test[$key] = $token == 'param' ? $tokens[++$i] : true;
+            }
+            else if ($token && preg_match('/^:index$/i', $token)) {
                 $test['index'] = intval($tokens[++$i]);
                 if ($tokens[$i+1] && preg_match('/^:last$/i', $tokens[$i+1])) {
                     $test['last'] = true;
                     $i++;
                 }
-           }
-           else {
-               $result[] = $tokens[$i];
-           }
+            }
+            else {
+                $result[] = $tokens[$i];
+            }
         }
 
         $tokens = $result;
@@ -1050,10 +1199,10 @@ class rcube_sieve_script
     /**
      * Extract action arguments
      */
-    private function action_arguments(&$tokens, $bool_args, $val_args = array())
+    private function action_arguments(&$tokens, $bool_args, $val_args = [])
     {
-        $action = array();
-        $result = array();
+        $action = [];
+        $result = [];
 
         for ($i=0, $len=count($tokens); $i<$len; $i++) {
             $tok = $tokens[$i];
@@ -1090,8 +1239,9 @@ class rcube_sieve_script
     static function escape_string($str)
     {
         if (is_array($str) && count($str) > 1) {
-            foreach ($str as $idx => $val)
+            foreach ($str as $idx => $val) {
                 $str[$idx] = self::escape_string($val);
+            }
 
             return '[' . implode(',', $str) . ']';
         }
@@ -1100,8 +1250,8 @@ class rcube_sieve_script
         }
 
         // multi-line string
-        if (preg_match('/[\r\n\0]/', $str) || strlen($str) > 1024) {
-            return sprintf("text:\n%s\n.\n", self::escape_multiline_string($str));
+        if (preg_match('/[\r\n\0]/', $str)) {
+            return sprintf("text:\r\n%s\r\n.\r\n", self::escape_multiline_string($str));
         }
         // quoted-string
         else {
@@ -1143,7 +1293,7 @@ class rcube_sieve_script
      */
     static function tokenize($str, $num = 0, &$position = 0)
     {
-        $result = array();
+        $result = [];
         $length = strlen($str);
 
         // remove spaces from the beginning of the string
@@ -1292,7 +1442,7 @@ class rcube_sieve_script
                         // remove dot-stuffing
                         $text = str_replace("\n..", "\n.", $text);
 
-                        $result[] = $text;
+                        $result[] = rtrim($text, "\r\n");
                         $position++;
                     }
                 }
@@ -1305,7 +1455,7 @@ class rcube_sieve_script
             }
         }
 
-        return $num === 1 ? (isset($result[0]) ? $result[0] : null) : $result;
+        return $num === 1 ? ($result[0] ?? null) : $result;
     }
 
     /**
@@ -1313,7 +1463,7 @@ class rcube_sieve_script
      */
     static function ltrim_position($content, $position, $br = true)
     {
-        $blanks = array("\t", "\0", "\x0B", " ");
+        $blanks = ["\t", "\0", "\x0B", " "];
 
         if ($br) {
             $blanks[] = "\r";
