@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  | Copyright (c) 2005-2007, Jon Abernathy <jon@chuggnutt.com>            |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
@@ -97,6 +98,11 @@
  */
 class rcube_html2text
 {
+    const LINKS_NONE = 0;
+    const LINKS_END = 1;
+    const LINKS_INLINE = 2;
+    const LINKS_DEFAULT = self::LINKS_END;
+
     /**
      * Contains the HTML content to convert.
      *
@@ -117,7 +123,7 @@ class rcube_html2text
      * Set this value to 0 (or less) to ignore word wrapping
      * and not constrain text to a fixed-width column.
      *
-     * @var integer $width
+     * @var int $width
      */
     protected $width = 70;
 
@@ -133,11 +139,10 @@ class rcube_html2text
      * used in conjunction with $replace.
      *
      * @var array $search
-     * @see $replace
+     * @see self::$replace
      */
-    protected $search = array(
+    protected $search = [
         '/\r/',                                  // Non-legal carriage return
-        '/^.*<body[^>]*>\n*/is',                 // Anything before <body>
         '/<head[^>]*>.*?<\/head>/is',            // <head>
         '/<script[^>]*>.*?<\/script>/is',        // <script>
         '/<style[^>]*>.*?<\/style>/is',          // <style>
@@ -157,17 +162,16 @@ class rcube_html2text
         '/(<table[^>]*>|<\/table>)/i',           // <table> and </table>
         '/(<tr[^>]*>|<\/tr>)/i',                 // <tr> and </tr>
         '/<td[^>]*>(.*?)<\/td>/i',               // <td> and </td>
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched.
      *
      * @var array $replace
-     * @see $search
+     * @see self::$search
      */
-    protected $replace = array(
+    protected $replace = [
         '',                                     // Non-legal carriage return
-        '',                                     // Anything before <body>
         '',                                     // <head>
         '',                                     // <script>
         '',                                     // <style>
@@ -187,19 +191,18 @@ class rcube_html2text
         "\n\n",                                 // <table> and </table>
         "\n",                                   // <tr> and </tr>
         "\t\t\\1\n",                            // <td> and </td>
-    );
+    ];
 
     /**
      * List of preg* regular expression patterns to search for,
      * used in conjunction with $ent_replace.
      *
      * @var array $ent_search
-     * @see $ent_replace
+     * @see self::$ent_replace
      */
-    protected $ent_search = array(
+    protected $ent_search = [
         '/&(nbsp|#160);/i',                      // Non-breaking space
-        '/&(quot|rdquo|ldquo|#8220|#8221|#147|#148);/i',
-                                         // Double quotes
+        '/&(quot|rdquo|ldquo|#8220|#8221|#147|#148);/i', // Double quotes
         '/&(apos|rsquo|lsquo|#8216|#8217);/i',   // Single quotes
         '/&gt;/i',                               // Greater-than
         '/&lt;/i',                               // Less-than
@@ -213,15 +216,15 @@ class rcube_html2text
         '/&(euro|#8364);/i',                     // Euro sign
         '/&(amp|#38);/i',                        // Ampersand: see _converter()
         '/[ ]{2,}/',                             // Runs of spaces, post-handling
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched.
      *
      * @var array $ent_replace
-     * @see $ent_search
+     * @see self::$ent_search
      */
-    protected $ent_replace = array(
+    protected $ent_replace = [
         "\xC2\xA0",                             // Non-breaking space
         '"',                                    // Double quotes
         "'",                                    // Single quotes
@@ -237,7 +240,7 @@ class rcube_html2text
         'EUR',                                  // Euro sign. €
         '|+|amp|+|',                            // Ampersand: see _converter()
         ' ',                                    // Runs of spaces, post-handling
-    );
+    ];
 
     /**
      * List of preg* regular expression patterns to search for
@@ -245,48 +248,53 @@ class rcube_html2text
      *
      * @var array $callback_search
      */
-    protected $callback_search = array(
+    protected $callback_search = [
         '/<(a) [^>]*href=("|\')([^"\']+)\2[^>]*>(.*?)<\/a>/i', // <a href="">
         '/<(h)[123456]( [^>]*)?>(.*?)<\/h[123456]>/i',         // h1 - h6
-        '/<(b)( [^>]*)?>(.*?)<\/b>/i',                         // <b>
-        '/<(strong)( [^>]*)?>(.*?)<\/strong>/i',               // <strong>
         '/<(th)( [^>]*)?>(.*?)<\/th>/i',                       // <th> and </th>
-    );
+    ];
 
-   /**
-    * List of preg* regular expression patterns to search for in PRE body,
-    * used in conjunction with $pre_replace.
-    *
-    * @var array $pre_search
-    * @see $pre_replace
-    */
-    protected $pre_search = array(
+    /**
+     * List of preg* regular expression patterns to search for in PRE body,
+     * used in conjunction with $pre_replace.
+     *
+     * @var array $pre_search
+     * @see self::$pre_replace
+     */
+    protected $pre_search = [
         "/\n/",
         "/\t/",
         '/ /',
         '/<pre[^>]*>/',
         '/<\/pre>/'
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched for PRE body.
      *
      * @var array $pre_replace
-     * @see $pre_search
+     * @see self::$pre_search
      */
-    protected $pre_replace = array(
+    protected $pre_replace = [
         '<br>',
         '&nbsp;&nbsp;&nbsp;&nbsp;',
         '&nbsp;',
         '',
         ''
-    );
+    ];
+
+    /**
+     * Temp. PRE content
+     *
+     * @var string $pre_content
+     */
+    protected $pre_content = '';
 
     /**
      * Contains a list of HTML tags to allow in the resulting text.
      *
      * @var string $allowed_tags
-     * @see set_allowed_tags()
+     * @see self::set_allowed_tags()
      */
     protected $allowed_tags = '';
 
@@ -300,8 +308,9 @@ class rcube_html2text
     /**
      * Indicates whether content in the $html variable has been converted yet.
      *
-     * @var boolean $_converted
-     * @see $html, $text
+     * @var bool $_converted
+     * @see self::$html
+     * @see self::$text
      */
     protected $_converted = false;
 
@@ -309,17 +318,19 @@ class rcube_html2text
      * Contains URL addresses from links to be rendered in plain text.
      *
      * @var array $_link_list
-     * @see _build_link_list()
+     * @see self::_build_link_list()
      */
-    protected $_link_list = array();
+    protected $_link_list = [];
 
     /**
-     * Boolean flag, true if a table of link URLs should be listed after the text.
+     * Links handling.
+     * - 0 if links should be removed
+     * - 1 if a table of link URLs should be listed after the text
+     * - 2 if the link should be displayed to the original point in the text they appeared
      *
-     * @var boolean $_do_links
-     * @see __construct()
+     * @var int $_links_mode
      */
-    protected $_do_links = true;
+    protected $_links_mode = 1;
 
     /**
      * Constructor.
@@ -328,29 +339,50 @@ class rcube_html2text
      * will instantiate with that source propagated, all that has
      * to be done it to call get_text().
      *
-     * @param string  $source    HTML content
-     * @param boolean $from_file Indicates $source is a file to pull content from
-     * @param boolean $do_links  Indicate whether a table of link URLs is desired
-     * @param integer $width     Maximum width of the formatted text, 0 for no limit
+     * @param string   $source     HTML content
+     * @param bool     $from_file  Indicates $source is a file to pull content from
+     * @param bool|int $links_mode Links handling mode
+     * @param int      $width      Maximum width of the formatted text, 0 for no limit
      */
-    function __construct($source = '', $from_file = false, $do_links = true, $width = 75, $charset = 'UTF-8')
+    function __construct($source = '', $from_file = false, $links_mode = self::LINKS_DEFAULT, $width = 75, $charset = 'UTF-8')
     {
         if (!empty($source)) {
             $this->set_html($source, $from_file);
         }
 
         $this->set_base_url();
+        $this->set_links_mode($links_mode);
 
-        $this->_do_links = $do_links;
-        $this->width     = $width;
-        $this->charset   = $charset;
+        $this->width   = $width;
+        $this->charset = $charset;
+    }
+
+    /**
+     * Sets the links behavior mode
+     *
+     * @param bool|int $mode
+     */
+    private function set_links_mode($mode)
+    {
+        $allowed = [
+            self::LINKS_NONE,
+            self::LINKS_END,
+            self::LINKS_INLINE
+        ];
+
+        if (!in_array((int) $mode, $allowed)) {
+            $this->_links_mode = self::LINKS_DEFAULT;
+            return;
+        }
+
+        $this->_links_mode = (int) $mode;
     }
 
     /**
      * Loads source HTML into memory, either from $source string or a file.
      *
-     * @param string  $source    HTML content
-     * @param boolean $from_file Indicates $source is a file to pull content from
+     * @param string $source    HTML content
+     * @param bool   $from_file Indicates $source is a file to pull content from
      */
     function set_html($source, $from_file = false)
     {
@@ -427,7 +459,7 @@ class rcube_html2text
     protected function _convert()
     {
         // Variables used for building the link list
-        $this->_link_list = array();
+        $this->_link_list = [];
 
         $text = $this->html;
 
@@ -464,11 +496,19 @@ class rcube_html2text
         // Convert <PRE>
         $this->_convert_pre($text);
 
+        // Remove body tag and anything before
+        // We used to have '/^.*<body[^>]*>\n*/is' in $this->search, but this requires
+        // high pcre.backtrack_limit setting when converting long HTML strings (#8137)
+        if (($pos = stripos($text, '<body')) !== false) {
+            $pos = strpos($text, '>', $pos);
+            $text = substr($text, $pos + 1);
+        }
+
         // Run our defined tags search-and-replace
         $text = preg_replace($this->search, $this->replace, $text);
 
         // Run our defined tags search-and-replace with callback
-        $text = preg_replace_callback($this->callback_search, array($this, 'tags_preg_callback'), $text);
+        $text = preg_replace_callback($this->callback_search, [$this, 'tags_preg_callback'], $text);
 
         // Strip any other HTML tags
         $text = strip_tags($text, $this->allowed_tags);
@@ -493,13 +533,13 @@ class rcube_html2text
         $text = preg_replace("/\n\s+\n/", "\n\n", $text);
         $text = preg_replace("/[\n]{3,}/", "\n\n", $text);
 
-        // remove leading empty lines (can be produced by eg. P tag on the beginning)
+        // remove leading empty lines (can be produced by e.g. P tag on the beginning)
         $text = ltrim($text, "\n");
 
         // Wrap the text to a readable format
         // for PHP versions >= 4.0.2. Default width is 75
         // If width is 0 or less, don't wrap the text.
-        if ( $this->width > 0 ) {
+        if ($this->width > 0) {
             $text = wordwrap($text, $this->width);
         }
     }
@@ -508,16 +548,16 @@ class rcube_html2text
      * Helper function called by preg_replace() on link replacement.
      *
      * Maintains an internal list of links to be displayed at the end of the
-     * text, with numeric indices to the original point in the text they
+     * text, with numeric indices or simply the link to the original point in the text they
      * appeared. Also makes an effort at identifying and handling absolute
      * and relative links.
      *
      * @param string $link    URL of the link
      * @param string $display Part of the text to associate number with
      */
-    protected function _build_link_list($link, $display)
+    protected function _handle_link($link, $display)
     {
-        if (!$this->_do_links || empty($link)) {
+        if (empty($link)) {
             return $display;
         }
 
@@ -542,6 +582,52 @@ class rcube_html2text
             $url .= "$link";
         }
 
+        if (self::LINKS_NONE === $this->_links_mode) {
+            // When not using link list use URL if there's no content (#5795)
+            // The content here is HTML, convert it to text first
+            $h2t     = new rcube_html2text($display, false, false, 1024, $this->charset);
+            $display = $h2t->get_text();
+
+            if (empty($display) && preg_match('!^([a-z][a-z0-9.+-]+://)!i', $link)) {
+                return $link;
+            }
+
+            return $display;
+        }
+
+        if (self::LINKS_INLINE === $this->_links_mode) {
+            return $this->_build_link_inline($url, $display);
+        }
+
+        return $this->_build_link_list($url, $display);
+    }
+
+    /**
+     * Helper function called by _handle_link() on link replacement.
+     *
+     * Displays the link next to the original point in the text they
+     * appeared.
+     *
+     * @param string $url     URL of the link
+     * @param string $display linktext
+     */
+    protected function _build_link_inline($url, $display)
+    {
+        return $display . ' &lt;' . $url . '&gt;';
+    }
+
+    /**
+     * Helper function called by _handle_link() on link replacement.
+     *
+     * Maintains an internal list of links to be displayed at the end of the
+     * text, with numeric indices to the original point in the text they
+     * appeared.
+     *
+     * @param string $url    URL of the link
+     * @param string $display Part of the text to associate number with
+     */
+    protected function _build_link_list($url, $display)
+    {
         if (($index = array_search($url, $this->_link_list)) === false) {
             $index = count($this->_link_list);
             $this->_link_list[] = $url;
@@ -563,7 +649,7 @@ class rcube_html2text
 
             // Run our defined tags search-and-replace with callback
             $this->pre_content = preg_replace_callback($this->callback_search,
-                array($this, 'tags_preg_callback'), $this->pre_content);
+                [$this, 'tags_preg_callback'], $this->pre_content);
 
             // convert the content
             $this->pre_content = sprintf('<div><br>%s<br></div>',
@@ -571,7 +657,7 @@ class rcube_html2text
 
             // replace the content (use callback because content can contain $0 variable)
             $text = preg_replace_callback('/<pre[^>]*>.*<\/pre>/ismU',
-                array($this, 'pre_preg_callback'), $text, 1);
+                [$this, 'pre_preg_callback'], $text, 1);
 
             // free memory
             $this->pre_content = '';
@@ -585,10 +671,12 @@ class rcube_html2text
      */
     protected function _convert_blockquotes(&$text)
     {
-        $level = 0;
+        $level  = 0;
         $offset = 0;
+
         while (($start = stripos($text, '<blockquote', $offset)) !== false) {
             $offset = $start + 12;
+
             do {
                 $end = stripos($text, '</blockquote>', $offset);
                 $next = stripos($text, '<blockquote', $offset);
@@ -618,12 +706,12 @@ class rcube_html2text
                     // replace content with inner blockquotes
                     $this->_converter($body);
 
-                    // resore text width
+                    // restore text width
                     $this->width = $p_width;
 
                     // Add citation markers and create <pre> block
-                    $body = preg_replace_callback('/((?:^|\n)>*)([^\n]*)/', array($this, 'blockquote_citation_callback'), trim($body));
-                    $body = '<pre>' . htmlspecialchars($body) . '</pre>';
+                    $body = preg_replace_callback('/((?:^|\n)>*)([^\n]*)/', [$this, 'blockquote_citation_callback'], trim($body));
+                    $body = '<pre>' . htmlspecialchars($body, ENT_COMPAT | ENT_HTML401 | ENT_SUBSTITUTE, $this->charset) . '</pre>';
 
                     $text = substr_replace($text, $body . "\n", $start, $end + 13 - $start);
                     $offset = 0;
@@ -645,7 +733,7 @@ class rcube_html2text
     public function blockquote_citation_callback($m)
     {
         $line  = ltrim($m[2]);
-        $space = $line[0] == '>' ? '' : ' ';
+        $space = isset($line[0]) && $line[0] == '>' ? '' : ' ';
 
         return $m[1] . '>' . $space . $line;
     }
@@ -654,14 +742,12 @@ class rcube_html2text
      * Callback function for preg_replace_callback use.
      *
      * @param array $matches PREG matches
-     * @return string
+     *
+     * @return string Element content
      */
     public function tags_preg_callback($matches)
     {
         switch (strtolower($matches[1])) {
-        case 'b':
-        case 'strong':
-            return $this->_toupper($matches[3]);
         case 'th':
             return $this->_toupper("\t\t". $matches[3] ."\n");
         case 'h':
@@ -669,7 +755,7 @@ class rcube_html2text
         case 'a':
             // Remove spaces in URL (#1487805)
             $url = str_replace(' ', '', $matches[3]);
-            return $this->_build_link_list($url, $matches[4]);
+            return $this->_handle_link($url, $matches[4]);
         }
     }
 
@@ -677,7 +763,8 @@ class rcube_html2text
      * Callback function for preg_replace_callback use in PRE content handler.
      *
      * @param array $matches PREG matches
-     * @return string
+     *
+     * @return string PRE content
      */
     public function pre_preg_callback($matches)
     {
@@ -688,12 +775,13 @@ class rcube_html2text
      * Strtoupper function with HTML tags and entities handling.
      *
      * @param string $str Text to convert
+     *
      * @return string Converted text
      */
     private function _toupper($str)
     {
         // string can containing HTML tags
-        $chunks = preg_split('/(<[^>]*>)/', $str, null, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE);
+        $chunks = preg_split('/(<[^>]*>)/', $str, -1, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE);
 
         // convert toupper only the text between HTML tags
         foreach ($chunks as $idx => $chunk) {
@@ -709,6 +797,7 @@ class rcube_html2text
      * Strtoupper multibyte wrapper function with HTML entities handling.
      *
      * @param string $str Text to convert
+     *
      * @return string Converted text
      */
     private function _strtoupper($str)
