@@ -31,10 +31,10 @@ class rcube_string_replacer
     public $linkref_index;
     public $linkref_pattern;
 
-    protected $values   = array();
-    protected $options  = array();
-    protected $linkrefs = array();
-    protected $urls     = array();
+    protected $values   = [];
+    protected $options  = [];
+    protected $linkrefs = [];
+    protected $urls     = [];
     protected $noword   = '[^\w@.#-]';
 
 
@@ -43,7 +43,7 @@ class rcube_string_replacer
      *
      * @param array $options Configuration options
      */
-    function __construct($options = array())
+    function __construct($options = [])
     {
         // Create hard-to-guess replacement string
         $uniq_ident    = sprintf('%010d%010d', mt_rand(), mt_rand());
@@ -103,10 +103,12 @@ class rcube_string_replacer
      * @return string Return valid link for recognized schemes, otherwise
      *                return the unmodified URL.
      */
-    public function link_callback($matches)
+    protected function link_callback($matches)
     {
-        $i = -1;
-        $scheme = strtolower($matches[1]);
+        $i          = -1;
+        $scheme     = strtolower($matches[1]);
+        $url_prefix = '';
+        $prefix     = '';
 
         if (preg_match('!^(http|ftp|file)s?://!i', $scheme)) {
             $url = $matches[1] . $matches[2];
@@ -117,9 +119,9 @@ class rcube_string_replacer
             $prefix     = $m[1];
         }
 
-        if ($url) {
+        if (!empty($url)) {
             $suffix = $this->parse_url_brackets($url);
-            $attrib = (array)$this->options['link_attribs'];
+            $attrib = isset($this->options['link_attribs']) ? (array) $this->options['link_attribs'] : [];
             $attrib['href'] = $url_prefix . $url;
 
             $i = $this->add(html::a($attrib, rcube::Q($url)) . $suffix);
@@ -132,35 +134,53 @@ class rcube_string_replacer
     /**
      * Callback to add an entry to the link index
      *
-     * @param array $matches Matches result from preg_replace_callback
+     * @param array $matches Matches result from preg_replace_callback with PREG_OFFSET_CAPTURE
      *
      * @return string Replacement string
      */
-    public function linkref_addindex($matches)
+    protected function linkref_addindex($matches)
     {
-        $key = $matches[1];
-        $this->linkrefs[$key] = isset($this->urls[$matches[3]]) ? $this->urls[$matches[3]] : null;
+        $key = $matches[1][0];
 
-        return $this->get_replacement($this->add('['.$key.']')) . $matches[2];
+        if (!isset($this->linkrefs[$key])) {
+            $this->linkrefs[$key] = [];
+        }
+
+        // Store the reference and its occurrence position
+        $this->linkrefs[$key][] = [
+            $this->urls[$matches[3][0]] ?? null,
+            $matches[0][1]
+        ];
+
+        return $this->get_replacement($this->add('['.$key.']')) . $matches[2][0];
     }
 
     /**
      * Callback to replace link references with real links
      *
-     * @param array $matches Matches result from preg_replace_callback
+     * @param array $matches Matches result from preg_replace_callback with PREG_OFFSET_CAPTURE
      *
      * @return string Replacement string
      */
-    public function linkref_callback($matches)
+    protected function linkref_callback($matches)
     {
         $i = 0;
-        if ($url = $this->linkrefs[$matches[1]]) {
-            $attrib = (array)$this->options['link_attribs'];
-            $attrib['href'] = $url;
-            $i = $this->add(html::a($attrib, rcube::Q($matches[1])));
+        $key = $matches[1][0];
+
+        if (!empty($this->linkrefs[$key])) {
+            $attrib = isset($this->options['link_attribs']) ? (array) $this->options['link_attribs'] : [];
+
+            foreach ($this->linkrefs[$key] as $linkref) {
+                $attrib['href'] = $linkref[0];
+                if ($linkref[1] >= $matches[1][1]) {
+                    break;
+                }
+            }
+
+            $i = $this->add(html::a($attrib, rcube::Q($matches[1][0])));
         }
 
-        return $i > 0 ? '['.$this->get_replacement($i).']' : $matches[0];
+        return $i > 0 ? '[' . $this->get_replacement($i) . ']' : $matches[0][0];
     }
 
     /**
@@ -170,7 +190,7 @@ class rcube_string_replacer
      *
      * @return string Replacement string
      */
-    public function mailto_callback($matches)
+    protected function mailto_callback($matches)
     {
         $href   = $matches[1];
         $suffix = $this->parse_url_brackets($href);
@@ -187,9 +207,9 @@ class rcube_string_replacer
      *
      * @return string Value at index $matches[1]
      */
-    public function replace_callback($matches)
+    protected function replace_callback($matches)
     {
-        return isset($this->values[$matches[1]]) ? $this->values[$matches[1]] : null;
+        return $this->values[$matches[1]] ?? null;
     }
 
     /**
@@ -201,12 +221,43 @@ class rcube_string_replacer
      */
     public function replace($str)
     {
+        if (!is_string($str)) {
+            return '';
+        }
+
         // search for patterns like links and e-mail addresses
-        $str = preg_replace_callback($this->link_pattern, array($this, 'link_callback'), $str);
-        $str = preg_replace_callback($this->mailto_pattern, array($this, 'mailto_callback'), $str);
+        $str = preg_replace_callback($this->link_pattern, [$this, 'link_callback'], $str);
+        $str = preg_replace_callback($this->mailto_pattern, [$this, 'mailto_callback'], $str);
+
         // resolve link references
-        $str = preg_replace_callback($this->linkref_index, array($this, 'linkref_addindex'), $str);
-        $str = preg_replace_callback($this->linkref_pattern, array($this, 'linkref_callback'), $str);
+/*
+        This code requires PHP 7.4 and could be used instead of the two if() statements below,
+        when we get there.
+
+        $str = preg_replace_callback($this->linkref_index,
+            [$this, 'linkref_addindex'], $str, -1, $count, PREG_OFFSET_CAPTURE
+        );
+        $str = preg_replace_callback($this->linkref_pattern,
+            [$this, 'linkref_callback'], $str, -1, $count, PREG_OFFSET_CAPTURE
+        );
+*/
+        if (preg_match_all($this->linkref_index, $str, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER)) {
+            $diff = 0;
+            foreach ($matches as $m) {
+                $replace = $this->linkref_addindex($m);
+                $str     = substr_replace($str, $replace, $m[0][1] + $diff, strlen($m[0][0]));
+                $diff   += strlen($replace) - strlen($m[0][0]);
+            }
+        }
+
+        if (preg_match_all($this->linkref_pattern, $str, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER)) {
+            $diff = 0;
+            foreach ($matches as $m) {
+                $replace = $this->linkref_callback($m);
+                $str     = substr_replace($str, $replace, $m[0][1] + $diff, strlen($m[0][0]));
+                $diff   += strlen($replace) - strlen($m[0][0]);
+            }
+        }
 
         return $str;
     }
@@ -220,7 +271,7 @@ class rcube_string_replacer
      */
     public function resolve($str)
     {
-        return preg_replace_callback($this->pattern, array($this, 'replace_callback'), $str);
+        return preg_replace_callback($this->pattern, [$this, 'replace_callback'], $str);
     }
 
     /**
@@ -240,17 +291,21 @@ class rcube_string_replacer
         // Yes, this is not perfect handles correctly only paired characters
         // but it should work for common cases
 
+        $suffix = '';
+
         if (preg_match('/(\\[|\\])/', $url)) {
             $in = false;
             for ($i=0, $len=strlen($url); $i<$len; $i++) {
                 if ($url[$i] == '[') {
-                    if ($in)
+                    if ($in) {
                         break;
+                    }
                     $in = true;
                 }
                 else if ($url[$i] == ']') {
-                    if (!$in)
+                    if (!$in) {
                         break;
+                    }
                     $in = false;
                 }
             }
@@ -266,13 +321,15 @@ class rcube_string_replacer
             $in = false;
             for ($i=0, $len=strlen($url); $i<$len; $i++) {
                 if ($url[$i] == '(') {
-                    if ($in)
+                    if ($in) {
                         break;
+                    }
                     $in = true;
                 }
                 else if ($url[$i] == ')') {
-                    if (!$in)
+                    if (!$in) {
                         break;
+                    }
                     $in = false;
                 }
             }
